#include <errno.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <linux/types.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define TICFG_DEVNODE	"/dev/ticfg"

#define TICFG_IOCTL_MAGIC       0xde

#define TICFG_IOCTL_ERASE               _IO(TICFG_IOCTL_MAGIC, 0)
#define TICFG_IOCTL_FLAG_READ_PRIMARY   _IOWR(TICFG_IOCTL_MAGIC, 1, unsigned int)
#define TICFG_IOCTL_FLAG_SET_PRIMARY    _IOWR(TICFG_IOCTL_MAGIC, 2, unsigned int)
#define TICFG_IOCTL_FLAG_READ_SECONDARY _IOWR(TICFG_IOCTL_MAGIC, 3, unsigned int)
#define TICFG_IOCTL_FLAG_SET_SECONDARY  _IOWR(TICFG_IOCTL_MAGIC, 4, unsigned int)

__u32 get_flash_checksum(void)
{
	int tifd, i;
	__u32 csum;

	if ((tifd = open(TICFG_DEVNODE, O_RDWR)) == -1) {
		perror("open(" TICFG_DEVNODE ")");
		return 1;
	}

	/* the checksum is at byte[12:15] (inclusively, starting at zero) */
	for (i = 0; i < 3; i++) {
		if (read(tifd, &csum, 4) != 4) {
			perror("read()");
			close(tifd);
			return 0;
		}
	}

	/* read little endian checksum value */
	for (i = 0, csum = 0; i < 4; i++) {
		if (read(tifd, &((char *)&csum)[i], 1) != 1) {
			perror("read()");
			close(tifd);
			return 0;
		}
	}

	close(tifd);
	return csum;
}

__u32 get_str_checksum(const char *str, size_t len)
{
	if (len < 16)
		return 0;

	return (str[12] & 0xff) | ((str[13] & 0xff) << 8)
		| ((str[14] & 0xff) << 16) | ((str[15] & 0xff) << 24);
}

int flash_config_str(const char *cstr, size_t clen)
{
	int tifd;

	if ((tifd = open(TICFG_DEVNODE, O_RDWR)) == -1) {
		perror("open(" TICFG_DEVNODE ")");
		return 1;
	}

	if (ioctl(tifd, TICFG_IOCTL_ERASE)) {
		perror("ioctl()");
		/* try to continue even if flash_erase failed */
	}

	if (write(tifd, cstr, clen) != (signed)clen) {
		perror("write()");
		close(tifd);
		return 1;
	}

	close(tifd);
	return 0;
}

int flash_config(char *fname)
{
	int tifd, cfd, rc = 0;
	char c;

	if ((cfd = open(fname, O_RDONLY)) == -1) {
		perror("open()");
		return 1;
	}

	if ((tifd = open(TICFG_DEVNODE, O_RDWR)) == -1) {
		perror("open(" TICFG_DEVNODE ")");
		rc = tifd;
		goto err_out_cfd;
	}

	if (ioctl(tifd, TICFG_IOCTL_ERASE)) {
		perror("ioctl()");
		/* try to continue even if flash_erase failed */
	}

	while ((rc = read(cfd, &c, 1)) == 1) {
		if (write(tifd, &c, 1) != 1) {
			perror("write()");
			rc = 1;
			break;
		}
	}
	if (rc == -1)
		perror("read()");

	close(tifd);
err_out_cfd:
	close(cfd);
	return rc;
}

#ifdef __TICFG_TOOL_STANDALONE__
int main(int argc, char **argv)
{
	if (argc != 2) {
		printf("Usage: %s <config.bin>\n", argv[0]);
		return 1;
	}

	return flash_config(argv[1]);
}
#endif /* __TICFG_TOOL_STANDALONE__ */
